/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "rec_tech.h"
#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <libgnome-media-profiles/audio-profile.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <math.h>

#define DECAY_STEP .15

GtkWidget *level;
static gdouble last_input_peak;

static void
show_error_dialog (GtkWindow *win, const gchar *dbg, const gchar * format, ...)
{
	GtkWidget *dialog;
	va_list args;
	gchar *s;

	va_start (args, format);
	s = g_strdup_vprintf (format, args);
	va_end (args);

	dialog = gtk_message_dialog_new (win,
					 GTK_DIALOG_DESTROY_WITH_PARENT,
					 GTK_MESSAGE_ERROR,
					 GTK_BUTTONS_CLOSE,
					 "%s",
					 s);

	if (dbg != NULL) {
		g_printerr ("ERROR: %s\nDEBUG MESSAGE: %s\n", s, dbg);
	}

	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
	g_free (s);
}

static void
show_missing_known_element_error (GtkWindow *win, gchar *description,
	gchar *element, gchar *plugin, gchar *module)
{
	show_error_dialog (win, NULL,
		_("Could not create the GStreamer %s element.\n"
		  "Please install the '%s' plugin from the '%s' module.\n"
		  "Verify that the installation is correct by running\n"
		  "		gst-inspect-1.0 %s\n"
		  "and then restart gnomeradio."),
		description, plugin, module, element);
}

static void
show_profile_error (GtkWindow *win, gchar *debug, gchar *description,
	const char *profile)
{
	gchar *first;

	first = g_strdup_printf (description, profile);
	show_error_dialog (win, debug, "%s%s", first,
		_("Please verify its settings.\n"
		  "You may be missing the necessary plugins."));
	g_free (first);
}

static void pipeline_error_cb(GstBus *bus, GstMessage *message, gpointer user_data)
{
	GError *error = NULL;
	GstElement *pipeline = user_data;
	g_assert(pipeline);
	
	/* Make sure the pipeline is not running any more */
	gst_element_set_state (pipeline, GST_STATE_NULL);
	
	gst_message_parse_error(message, &error, NULL);
	g_warning(_("GStreamer runtime error: %s\n"), error->message);
	g_error_free(error);
}

static gboolean
level_message_cb (GstBus *bus, GstMessage *message, gpointer user_data)
{
	if (message->type == GST_MESSAGE_ELEMENT) {
		const GstStructure *s = gst_message_get_structure (message);
		const gchar *name = gst_structure_get_name (s);

		if (g_strcmp0 (name, "level") == 0) {
			gint channels;
			gdouble peak_dB;
			gdouble peak;
			const GValue *array_val;
			const GValue *value;

			GValueArray *rms_arr, *peak_arr;
			gint i;

			array_val = gst_structure_get_value (s, "rms");
			rms_arr = (GValueArray *) g_value_get_boxed (array_val);

			array_val = gst_structure_get_value (s, "peak");
			peak_arr = (GValueArray *) g_value_get_boxed (array_val);

			channels = rms_arr->n_values;

			for (i = 0; i < channels; ++i) {
				value = peak_arr->values + i;
				peak_dB = g_value_get_double (value);

				peak = pow (10, peak_dB / 20);
			}

			if (last_input_peak >= DECAY_STEP) {
				if (peak < last_input_peak - DECAY_STEP) {
					peak = last_input_peak - DECAY_STEP;
				}
			}

			last_input_peak = peak;

			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (level), peak);
		}
	}

	return TRUE;
}

Recording*
recording_start(const char* filename)
{
	GMAudioProfile *profile;
	const gchar *profile_pipeline_desc;
	GstElement *pipeline, *source, *encoder, *filesink, *level;
	pipeline = source = encoder = filesink = level = NULL;
	GError *err = NULL;
	gchar *pipeline_desc;
	const char *name;

	source = gst_element_factory_make ("autoaudiosrc", "autoaudiosource");
	if (source == NULL) {
		show_missing_known_element_error (NULL,
		    _("GConf audio recording"), "autoaudiosrc",
		      "gconfelements", "gst-plugins-good");
		return FALSE;
	}

	if (!gst_element_set_state (source, GST_STATE_READY)) {
		show_error_dialog (NULL, NULL,
		    _("Your audio capture settings are invalid. "
		      "Please correct them with the \"Sound Settings\" "
		      "under the System Settings menu."));
		return FALSE;
	}

	filesink = gst_element_factory_make ("filesink", "sink");
	if (filesink == NULL)
	{
		show_missing_known_element_error (NULL,
		    _("file output"), "filesink", "coreelements", "gstreamer");
		gst_object_unref (source);
		return NULL;
	}

	pipeline = gst_pipeline_new("gnomeradio-record-pipeline");

	gst_bin_add (GST_BIN (pipeline), source);

	level = gst_element_factory_make ("level", "level");
	if (level == NULL)
	{
		show_missing_known_element_error (NULL,
		    _("level"), "level", "level", "gstreamer");
		gst_object_unref (source);
		return NULL;
	}
	gst_element_set_name (level, "level");

	profile = gm_audio_profile_lookup(rec_settings.profile);
	g_assert(profile);
	if (profile == NULL)
		return NULL;
	profile_pipeline_desc = gm_audio_profile_get_pipeline (profile);
	name = gm_audio_profile_get_name (profile);

	GST_DEBUG ("encoder profile pipeline: '%s'", GST_STR_NULL (profile_pipeline_desc));

	pipeline_desc = g_strdup_printf ("audioconvert ! %s", profile_pipeline_desc);
	GST_DEBUG ("making encoder bin from description '%s'", pipeline_desc);
	encoder = gst_parse_bin_from_description (pipeline_desc, TRUE, &err);
	g_free (pipeline_desc);
	pipeline_desc = NULL;

	if (err) {
		show_profile_error (NULL, err->message,
		    _("Could not parse the '%s' audio profile. "), name);
		g_printerr ("Failed to create GStreamer encoder plugins [%s]: %s\n", profile_pipeline_desc, err->message);
		g_error_free (err);
		gst_object_unref (pipeline);
		gst_object_unref (filesink);
		g_free (pipeline);
		return NULL;
	}

	gst_bin_add (GST_BIN (pipeline), level);
	gst_bin_add (GST_BIN (pipeline), encoder);
	gst_bin_add (GST_BIN (pipeline), filesink);

	if (!(gst_element_link_many (source, level, encoder, NULL))) {
		show_profile_error (NULL, NULL,
		    _("Could not capture using the '%s' audio profile. "), name);
		gst_object_unref (pipeline);
		g_free (pipeline);
		return NULL;
	}

	if (!gst_element_link (encoder, filesink)) {
		show_profile_error (NULL, NULL,
		    _("Could not write to a file using the '%s' audio profile. "), name);
		gst_object_unref (pipeline);
		g_free (pipeline);
		return NULL;
	}

	GstBus *bus = gst_element_get_bus(pipeline);
	gst_bus_add_signal_watch(bus);

	g_signal_connect(G_OBJECT(bus), "message::error", G_CALLBACK(pipeline_error_cb), pipeline);
	g_signal_connect (G_OBJECT(bus), "message::element", G_CALLBACK (level_message_cb), pipeline);

	char* path = g_strdup_printf("%s.%s", filename, gm_audio_profile_get_extension(profile));	
	g_object_set(G_OBJECT(filesink), "location", path, NULL);
	
	gst_element_set_state(pipeline, GST_STATE_PLAYING);
	
	Recording *recording = g_malloc0(sizeof(Recording));
	recording->filename = path;
	recording->pipeline = pipeline;
	
	return recording;
}		

void
recording_stop(Recording *recording)
{
	g_assert(recording);

	GstState state;
	gst_element_get_state(recording->pipeline, &state, NULL, GST_CLOCK_TIME_NONE);
	if (state != GST_STATE_PLAYING) {
		GST_DEBUG ("pipeline in wrong state: %s", gst_element_state_get_name (state));
	} else {
		GstMessage *msg;
		GST_DEBUG ("Stopping recording source");
		gst_element_send_event (recording->pipeline, gst_event_new_eos ());
		/* wait for EOS message on the pipeline bus */
		msg = gst_bus_timed_pop_filtered (GST_ELEMENT_BUS (recording->pipeline), GST_SECOND, GST_MESSAGE_EOS | GST_MESSAGE_ERROR);
		gst_message_unref (msg);

		gst_element_set_state(recording->pipeline, GST_STATE_NULL);
	}
	gst_object_unref(GST_OBJECT(recording->pipeline));
	g_free(recording->filename);
	g_free(recording->station);
	g_free(recording);
}	

/*
 * Miscellanelous functions
 */

int get_file_size(char *filename)
{
	struct stat buffer;
	g_assert(filename);
	
	if (lstat(filename, &buffer))
		return -1;
	return (int)buffer.st_size;
}

int check_filename(const char *filename)
{
	int flags, retval;
	
	g_assert(filename);
	
	flags = O_RDWR | O_CREAT | O_APPEND;
	
	retval = open(filename, flags, 0664);
	
	if (retval < 0)
	{
		return 0;
	}

	close(retval);
	return 1;
}
